@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2024 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2025 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 
 /* ImLib2 Header */
 #include <X11/Xlib.h>       /* needed by Imlib2.h */
@@ -27,8 +27,11 @@
 /* string manipulation */
 #include <string.h>         /* strcasecmp, strcmp, strrchr */
 
-/* sin, cos */
-#include <math.h>
+/* trigonometry */
+#include <math.h>           /* sin, cos, M_PI */
+#ifndef M_PI                /* sometimes, M_PI is not defined */
+#define M_PI 3.14159265358979323846
+#endif
 
 /* my headers */
 #include "defines.h"        /* defines */
@@ -76,12 +79,12 @@ void ssocr_set_color(fg_bg_t color)
 }
 
 /* draw a fore- or background pixel */
-void draw_pixel(Imlib_Image *image, int x, int y, fg_bg_t color)
+void draw_fg_bg_pixel(Imlib_Image *image, int x, int y, fg_bg_t color)
 {
   Imlib_Image *current_image; /* save current image */
 
   current_image = imlib_context_get_image();
-  imlib_context_set_image(image);
+  imlib_context_set_image(*image);
   ssocr_set_color(color);
   imlib_image_draw_pixel(x,y,0);
   imlib_context_set_image(current_image);
@@ -90,13 +93,25 @@ void draw_pixel(Imlib_Image *image, int x, int y, fg_bg_t color)
 /* draw a foreground pixel */
 void draw_fg_pixel(Imlib_Image *image, int x, int y)
 {
-  draw_pixel(image, x, y, FG);
+  draw_fg_bg_pixel(image, x, y, FG);
 }
 
 /* draw a background pixel */
 void draw_bg_pixel(Imlib_Image *image, int x, int y)
 {
-  draw_pixel(image, x, y, BG);
+  draw_fg_bg_pixel(image, x, y, BG);
+}
+
+/* draw a pixel of a given color */
+void draw_color_pixel(Imlib_Image *image, int x, int y, Imlib_Color color)
+{
+  Imlib_Image *current_image; /* save current image */
+
+  current_image = imlib_context_get_image();
+  imlib_context_set_image(*image);
+  imlib_context_set_color(color.red, color.green, color.blue, color.alpha);
+  imlib_image_draw_pixel(x, y, 0);
+  imlib_context_set_image(current_image);
 }
 
 /* check if a pixel is set regarding current foreground/background colors */
@@ -165,9 +180,9 @@ Imlib_Image set_pixels_filter(Imlib_Image *source_image, double thresh,
       }
       /* set pixel if at least mask pixels around it are set */
       if(set_pixel >= mask) {
-        draw_fg_pixel(new_image, x, y);
+        draw_fg_pixel(&new_image, x, y);
       } else {
-        draw_bg_pixel(new_image, x, y);
+        draw_bg_pixel(&new_image, x, y);
       }
     }
   }
@@ -257,12 +272,6 @@ Imlib_Image keep_pixels_filter(Imlib_Image *source_image, double thresh,
   width = imlib_image_get_width();
   new_image = imlib_clone_image();
 
-  /* draw white (background) rectangle to clear new image */
-  imlib_context_set_image(new_image);
-  ssocr_set_color(BG);
-  imlib_image_draw_rectangle(0, 0, width, height);
-  imlib_context_set_image(*source_image);
-
   /* check for every pixel if it should be set in filtered image */
   for(x=0; x<width; x++) {
     for(y=0; y<height; y++) {
@@ -285,9 +294,9 @@ Imlib_Image keep_pixels_filter(Imlib_Image *source_image, double thresh,
       /* set pixel if at least mask pixels around it are set */
       /* mask = 1 keeps all pixels */
       if(set_pixel > mask) {
-        draw_fg_pixel(new_image, x, y);
+        draw_fg_pixel(&new_image, x, y);
       } else {
-        draw_bg_pixel(new_image, x, y);
+        draw_bg_pixel(&new_image, x, y);
       }
     }
   }
@@ -402,9 +411,9 @@ Imlib_Image dynamic_threshold(Imlib_Image *source_image,double t,luminance_t lt,
       lum = get_lum(&color, lt);
       thresh = get_threshold(source_image, t/100.0, lt, x-ww/2, y-ww/2, ww, wh);
       if(is_pixel_set(lum, thresh)) {
-        draw_fg_pixel(new_image, x, y);
+        draw_fg_pixel(&new_image, x, y);
       } else {
-        draw_bg_pixel(new_image, x, y);
+        draw_bg_pixel(&new_image, x, y);
       }
     }
   }
@@ -441,9 +450,9 @@ Imlib_Image make_mono(Imlib_Image *source_image, double thresh, luminance_t lt)
       imlib_image_query_pixel(x, y, &color);
       lum = get_lum(&color, lt);
       if(is_pixel_set(lum, thresh)) {
-        draw_fg_pixel(new_image, x, y);
+        draw_fg_pixel(&new_image, x, y);
       } else {
-        draw_bg_pixel(new_image, x, y);
+        draw_bg_pixel(&new_image, x, y);
       }
     }
   }
@@ -456,23 +465,27 @@ Imlib_Image make_mono(Imlib_Image *source_image, double thresh, luminance_t lt)
 }
 
 /* adapt threshold to image values values */
-double adapt_threshold(Imlib_Image *image, double thresh, luminance_t lt, int x,
-                       int y, int w, int h, unsigned int flags)
+double adapt_threshold(Imlib_Image *image, double thresh, luminance_t lt,
+                       unsigned int flags, int force_update)
 {
   double t = thresh;
-  if(!(flags & ABSOLUTE_THRESHOLD)) {
+  static int is_adapted = 0;
+  if(is_adapted && !force_update) {
+    fprintf(stderr, "threshold is already adjusted to image\n");
+  } else if(!(flags & ABSOLUTE_THRESHOLD)) {
     if(flags & DEBUG_OUTPUT)
       fprintf(stderr, "adjusting threshold to image: %f ->", t);
-    t = get_threshold(image, thresh/100.0, lt, x, y, w, h);
+    t = get_threshold(image, thresh/100.0, lt, 0, 0, -1, -1);
     if(flags & DEBUG_OUTPUT)
       fprintf(stderr, " %f\n", t);
     if(flags & DO_ITERATIVE_THRESHOLD) {
       if(flags & DEBUG_OUTPUT)
         fprintf(stderr, "doing iterative_thresholding: %f ->", t);
-      t = iterative_threshold(image, t, lt, x, y, w, h);
+      t = iterative_threshold(image, t, lt);
       if(flags & DEBUG_OUTPUT)
         fprintf(stderr, " %f\n", t);
     }
+    is_adapted = 1;
   }
   if((flags & VERBOSE) || (flags & DEBUG_OUTPUT)) {
     fprintf(stderr, "using threshold %.2f\n", t);
@@ -528,7 +541,7 @@ double get_threshold(Imlib_Image *source_image, double fraction, luminance_t lt,
 
 /* determine threshold by an iterative method */
 double iterative_threshold(Imlib_Image *source_image, double thresh,
-                           luminance_t lt, int x, int y, int w, int h)
+                           luminance_t lt)
 {
   Imlib_Image current_image; /* save image pointer */
   int height, width; /* image dimensions */
@@ -553,23 +566,13 @@ double iterative_threshold(Imlib_Image *source_image, double thresh,
   height = imlib_image_get_height();
   width = imlib_image_get_width();
 
-  /* special value -1 for width or height means image width/height */
-  if(w == -1) w = width;
-  if(h == -1) h = width;
-
-  /* assure valid coordinates */
-  if(x+w > width) x = width-w;
-  if(y+h > height) y = height-h;
-  if(x<0) x=0;
-  if(y<0) y=0;
-
   /* find the threshold value to differentiate between dark and light */
   do {
     thresh_lum = MAXRGB * new_thresh;
     old_thresh = new_thresh;
     size_black = sum_black = size_white = sum_white = 0;
-    for(xi=0; (xi<w) && (xi<width); xi++) {
-      for(yi=0; (yi<h) && (yi<height); yi++) {
+    for(xi=0; xi<width; xi++) {
+      for(yi=0; yi<height; yi++) {
         imlib_image_query_pixel(xi, yi, &color);
         lum = get_lum(&color, lt);
         if(lum <= thresh_lum) {
@@ -604,92 +607,39 @@ double iterative_threshold(Imlib_Image *source_image, double thresh,
   return new_thresh * 100;
 }
 
-/* get minimum lum value */
-double get_minval(Imlib_Image *source_image, int x, int y, int w, int h,
-                 luminance_t lt)
+/* get minimum and maximum lum values */
+void get_minmaxval(Imlib_Image *source_image, luminance_t lt,
+                   double *min, double *max)
 {
   Imlib_Image current_image; /* save image pointer */
-  int height, width; /* image dimensions */
+  int w, h; /* image dimensions */
   int xi,yi; /* iteration variables */
   Imlib_Color color; /* Imlib2 RGBA color structure */
-  int minval = MAXRGB;
   int lum = 0;
 
+  *min = MAXRGB;
+  *max = 0;
+
   /* save pointer to current image */
   current_image = imlib_context_get_image();
 
   /* get image dimensions */
   imlib_context_set_image(*source_image);
-  height = imlib_image_get_height();
-  width = imlib_image_get_width();
-
-  /* special value -1 for width or height means image width/height */
-  if(w == -1) w = width;
-  if(h == -1) h = width;
-
-  /* assure valid coordinates */
-  if(x+w > width) x = width-w;
-  if(y+h > height) y = height-h;
-  if(x<0) x=0;
-  if(y<0) y=0;
+  h = imlib_image_get_height();
+  w = imlib_image_get_width();
 
   /* find the minimum value in the image */
-  for(xi=0; (xi<w) && (xi<width); xi++) {
-    for(yi=0; (yi<h) && (yi<height); yi++) {
+  for(xi=0; xi<w; xi++) {
+    for(yi=0; yi<h; yi++) {
       imlib_image_query_pixel(xi, yi, &color);
       lum = clip(get_lum(&color, lt),0,255);
-      if(lum < minval) minval = lum;
+      if(lum < *min) *min = lum;
+      if(lum > *max) *max = lum;
     }
   }
 
   /* restore image from before function call */
   imlib_context_set_image(current_image);
-
-  return minval;
-}
-
-/* get maximum luminance value */
-double get_maxval(Imlib_Image *source_image, int x, int y, int w, int h,
-                 luminance_t lt)
-{
-  Imlib_Image current_image; /* save image pointer */
-  int height, width; /* image dimensions */
-  int xi,yi; /* iteration variables */
-  Imlib_Color color; /* Imlib2 RGBA color structure */
-  int lum = 0;
-  int maxval = 0;
-
-  /* save pointer to current image */
-  current_image = imlib_context_get_image();
-
-  /* get image dimensions */
-  imlib_context_set_image(*source_image);
-  height = imlib_image_get_height();
-  width = imlib_image_get_width();
-
-  /* special value -1 for width or height means image width/height */
-  if(w == -1) w = width;
-  if(h == -1) h = width;
-
-  /* assure valid coordinates */
-  if(x+w > width) x = width-w;
-  if(y+h > height) y = height-h;
-  if(x<0) x=0;
-  if(y<0) y=0;
-
-  /* find the minimum value in the image */
-  for(xi=0; (xi<w) && (xi<width); xi++) {
-    for(yi=0; (yi<h) && (yi<height); yi++) {
-      imlib_image_query_pixel(xi, yi, &color);
-      lum = clip(get_lum(&color, lt),0,255);
-      if(lum > maxval) maxval = lum;
-    }
-  }
-
-  /* restore image from before function call */
-  imlib_context_set_image(current_image);
-
-  return maxval;
 }
 
 /* draw a white (background) border around image, overwriting image contents
@@ -757,15 +707,11 @@ Imlib_Image shear(Imlib_Image *source_image, int offset)
     /* copy pixels */
     for(x=width-1; x>=shift; x--) {
       imlib_image_query_pixel(x-shift, y, &color_return);
-      imlib_context_set_image(new_image);
-      imlib_context_set_color(color_return.red, color_return.green,
-                              color_return.blue, color_return.alpha);
-      imlib_image_draw_pixel(x,y,0);
-      imlib_context_set_image(*source_image);
+      draw_color_pixel(&new_image, x, y, color_return);
     }
     /* fill with background */
     for(x=0; x<shift; x++) {
-      draw_bg_pixel(new_image, x, y);
+      draw_bg_pixel(&new_image, x, y);
     }
   }
 
@@ -806,14 +752,10 @@ Imlib_Image rotate(Imlib_Image *source_image, double theta)
       sy = (y-height/2) * cos(theta) - (x-width/2) * sin(theta) + height/2;
       if((sx >= 0) && (sx <= width) && (sy >= 0) && (sy <= height)) {
         imlib_image_query_pixel(sx, sy, &c);
-        imlib_context_set_image(new_image);
-        imlib_context_set_color(c.red, c.green, c.blue, c.alpha);
+        draw_color_pixel(&new_image, x, y, c);
       } else {
-        imlib_context_set_image(new_image);
-        ssocr_set_color(BG);
+        draw_bg_pixel(&new_image, x, y);
       }
-      imlib_image_draw_pixel(x,y,0);
-      imlib_context_set_image(*source_image);
     }
   }
 
@@ -847,20 +789,14 @@ Imlib_Image mirror(Imlib_Image *source_image, direction_t direction)
     for(x = width-1; x>=0; x--) {
       for(y = 0; y < height; y++) {
         imlib_image_query_pixel(width - 1 - x, y, &c);
-        imlib_context_set_image(new_image);
-        imlib_context_set_color(c.red, c.green, c.blue, c.alpha);
-        imlib_image_draw_pixel(x,y,0);
-        imlib_context_set_image(*source_image);
+        draw_color_pixel(&new_image, x, y, c);
       }
     }
   } else if(direction == VERTICAL) {
     for(x = 0; x < width; x++) {
       for(y = height-1; y >= 0; y--) {
         imlib_image_query_pixel(x, height - 1 - y, &c);
-        imlib_context_set_image(new_image);
-        imlib_context_set_color(c.red, c.green, c.blue, c.alpha);
-        imlib_image_draw_pixel(x,y,0);
-        imlib_context_set_image(*source_image);
+        draw_color_pixel(&new_image, x, y, c);
       }
     }
   }
@@ -935,9 +871,9 @@ Imlib_Image invert(Imlib_Image *source_image, double thresh, luminance_t lt)
       imlib_image_query_pixel(x, y, &color);
       lum = get_lum(&color, lt);
       if(is_pixel_set(lum, thresh)) {
-        draw_bg_pixel(new_image, x, y);
+        draw_bg_pixel(&new_image, x, y);
       } else {
-        draw_fg_pixel(new_image, x, y);
+        draw_fg_pixel(&new_image, x, y);
       }
     }
   }
